#ifdef ARC32_SERVO_DRIVER
;Project Lynxmotion Phoenix
;Description: Phoenix software
;Software version: V2.1 alfa
;Date: 15-04-2011
;Programmer: Jeroen Janssen (aka Xan)
;
; SSC Driver - Contains all of the code that knows anything about the Arc32
;		being used to drive the servos
; 		It also has timer functions as this may also change with drivers...
;
;====================================================================

;[GP PLAYER]
wGPSeqPtr		var word
abGPServoPins	var	byte(32)		; remember which pins are defined for this sequence
bGPCntServos	var	byte			; how many servos are used in this sequence
bGPServoNum		var	byte
bGPCntSteps		var	byte			; how many moves are in this sequence...
bGPStepNum		var	byte			; which step are we on.
awGPStepServoVals var	word(32)	; servo value for each pin in desired pulse width
swHSVal			var	sword			; converted to HServo value.
wGPStepTime		var	word			; How long this step will take

; [Servo Driver variables]
;[HServo - or SSC32?]
StepsPerDegree 		con 200
aswCoxaServo	var sword(6)	; pre-calculated HSERVO values from Angles above
aswFemurServo	var sword(6)
aswTibiaServo	var sword(6)
fFirstMove		var bit			; need to handle first move specially...

; define the offset in the ARC32 EEPROM where we emulate the SSC-32 from
ARC32_SSC_OFFSET	con	0x400		; we will offset saving all of the SSC-32 by this amount
SERVOSAVECNT	con	32				
aServoOffsets	var	sword(SERVOSAVECNT)		; Our new values - must take stored away values into account...
bCSIn		var		byte					; used in Read/Write servo offsets - checksum read
bCSCalc		var		byte					; Used in Read/Write Servo offsets - calculated checksum



;--------------------------------------------------------------------
;[TIMER INTERRUPT INIT]
#ifndef BASICATOMPROARC32
Error: This only works on Arc32
#endif
  WTIMERTICSPERMSMUL con 256	; Arc32 is 20mhz need a multiplyer and divider to make the conversion with /8192
  WTIMERTICSPERMSDIV con 625  ; 



;--------------------------------------------------------------------
;[InitServoDriver] - Initializes the servo driver including the main timer used in the phoenix code
InitServoDriver:
; Arc32 will use timer associated with HSERVO

  ; DEBUG: setup 
  sethserial1 H38400

; Read in servo offsets.
  gosub ReadServoOffsets
  
  gosub FreeServos

  return


;==============================================================================
; Subroutine: ReadServoOffsets
; Will read in the zero points that wer last saved for the different servos
; that are part of this robot.  
;
;==============================================================================
ReadServoOffsets:
	readdm 31, [ bCSIn]
	readdm 32, [str aServoOffsets\32]	; We are storing words now.
	readdm 64, [str aServoOffsets(16)\32]
	bCSCalc = 0
	for Index = 0 to SERVOSAVECNT-1
		bCSCalc = bCSCalc + AServoOffsets(Index).lowbyte + AServoOffsets(Index).highbyte
	next
		
	if bCSCalc <> bCSIn then 
      	Sound cSpeakerPin,[60\4000,100\5000, 60\4000]	' make some noise...
		aServoOffsets = rep 0\SERVOSAVECNT
	endif

	return



;==============================================================================
;[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
;                makes sure of consistancy. That is it is very posible that 
;                after we grabed the timers value it overflows, before we grab the other part
;                so we check to make sure it is correct and if necesary regrab things.
;==============================================================================
GetCurrentTime:
  lCurrentTime = HSERVOTIME 13	; Clock / 8192
  return lCurrentTime


;-------------------------------------------------------------------------------------
;[ConvertTimeMS]
_ttconv	var	long
ConvertTimeMS[_ttconv]:
	return (_ttconv * WTIMERTICSPERMSMUL)/WTIMERTICSPERMSDIV 


;--------------------------------------------------------------------

;[CheckGPEnable] Checks to see if the driver supports General Purpose sequences
CheckGPEnable:
	
  pause 10
  GPEnable=1
  
  return 1		; We roll our own...

;--------------------------------------------------------------------
;[GP PLAYER]
GPPlayer:
  ;Start sequence
  IF (GPStart=1) THEN
    readdm ARC32_SSC_OFFSET + GPSEQ*2, [str wGPSeqPtr\2]
	hserout 1, ["GP: ", dec GPSEQ, 13]
    ; make sure there is a sequence...
    IF (wGPSeqPtr <> 0)  and (wGPSeqPtr <> 0xffff)	THEN
	  ; now lets get the header information, will skip sequence number
	  readdm wGPSeqPtr+1,[bGPCntServos, bGPCntSteps]

	  ; now lets read in the pins
	  wGPSeqPtr = wGPSeqPtr + 3 ; point to start of pin information.
	  for bGPServoNum = 0 to bGPCntServos - 1
		readdm wGPSeqPtr, [abGPServoPins(bGPServoNum)]	; read in the pin, igore max value after
		wGPSeqPtr = wGPSeqPtr + 3
	  next


	  ; now lets start cycling through the steps... Will do seperate HSERVO call for each pin in each step...
	  wGPSeqPtr = wGPSeqPtr + 2 ; ignore step-1 time.


  	  for bGPStepNum = 1 to bGPCntSteps ; start off 1 biased as I don't use this value and don't have to cnt -1...
#if 0
  	    ; Let the Controller know what step we are running... 
        gosub ControlGPStatus[bGPStepNum]
#endif
		hserout 1, ["Step:", dec bGPStepNum]
	    readdm wGPSeqPtr, [str awGPStepServoVals\2*bGPCntServos, wGPStepTime.highbyte, wGPStepTime.lowByte] ; note did not change time bytes order ???
	
	    for bGPServoNum = 0 to bGPCntServos - 1
		  ; need to convert the value to HServo Value
		  swHSVal = (awGPStepServoVals(bGPServoNum)*20 - 30000) +  aServoOffsets(abGPServoPins(bGPServoNum)) ; convert to hservo and take care of offsets
		  hservo [abGPServoPins(bGPServoNum) \ swHSVal\ abs(HServoPos(abGPServoPins(bGPServoNum)) - swHSVal) * 20 / wGPStepTime]
  		  hserout 1, [" ", dec abGPServoPins(bGPServoNum), ":", sdec swHSVal]
        next
	    ; now lets wait for the step to complete
	    ; Keep from hanging...
	    if wGPStepTime > 2000 or wGPStepTime = 0 then 
	      Sound cSpeakerPin,[60\4000,100\5000, 60\4000]	' make some noise...
	      wGPStepTime = 2000
	    endif
	    hserout 1, [" T:", dec wGPStepTime, 13]
	    pause wGPStepTime
	    wGPSeqPtr = wGPSeqPtr + 2*bGPCntServos + 2	; increment to the next sequence
	  next
    endif
	GPStart = 0	; say we are done	
  endif	
			
return
	
	
;--------------------------------------------------------------------
;[SERVO DRIVER] Updates the positions of the servos
;**********************************************************************
UpdateServoDriver:
  ;Update Right Legs
  ; BUGBUG : should we convert the offsets to their own table that has been pre multiplied?...
  for LegIndex = 0 to 2
    aswCoxaServo(LegIndex) = (-CoxaAngle1(LegIndex) * StepsPerDegree ) / 10  + aServoOffsets(cCoxaPin(LegIndex))	; Convert angle to HServo value
    aswFemurServo(LegIndex) = (-FemurAngle1(LegIndex) * StepsPerDegree ) / 10 + aServoOffsets(cFemurPin(LegIndex))
    aswTibiaServo(LegIndex) = (-TibiaAngle1(LegIndex) * StepsPerDegree ) / 10 + aServoOffsets(cTibiaPin(LegIndex))
  next
  
  ;Update Left Legs
  for LegIndex = 3 to 5
    aswCoxaServo(LegIndex) = (CoxaAngle1(LegIndex) * StepsPerDegree ) / 10 + aServoOffsets(cCoxaPin(LegIndex))
    aswFemurServo(LegIndex) = (FemurAngle1(LegIndex) * StepsPerDegree ) / 10	+ aServoOffsets(cFemurPin(LegIndex))
    aswTibiaServo(LegIndex) = (TibiaAngle1(LegIndex) * StepsPerDegree ) / 10 + aServoOffsets(cTibiaPin(LegIndex))
  next  

  PrevSSCTime = SSCTime
return
;--------------------------------------------------------------------
;[FREE SERVOS] Frees all the servos (binary)
FreeServos
  for LegIndex = 0 to 5
	hservo [cCoxaPin(LegIndex)	\	-30000, |
			cFemurPin(LegIndex)	\	-30000, |
			cTibiaPin(LegIndex)	\	-30000]
  next
  fFirstMove = 1		; May need to tell servo system again to do it without time values...
return

;--------------------------------------------------------------------
;[COMMIT SERVO POSITIONS]
CommitServoDriver:
  if fFirstMove then
	fFirstMove = 0		; Clear it...
    for LegIndex = 0 to 5
      ; first time through move imdediate as no real position to base off of...
      hservo [cCoxaPin(LegIndex)	\	aswCoxaServo(LegIndex), |
	          cFemurPin(LegIndex)	\	aswFemurServo(LegIndex), |
		      cTibiaPin(LegIndex)	\	aswTibiaServo(LegIndex)]
    next

  else
    for LegIndex = 0 to 5
      hservo [cCoxaPin(LegIndex)	\	aswCoxaServo(LegIndex)	\	abs(HServoPos(cCoxaPin(LegIndex)) - aswCoxaServo(LegIndex)) * 20 / SSCTime, |
	          cFemurPin(LegIndex)	\	aswFemurServo(LegIndex)	\	abs(HServoPos(cFemurPin(LegIndex)) - aswFemurServo(LegIndex)) * 20 / SSCTime, |
		      cTibiaPin(LegIndex)	\	aswTibiaServo(LegIndex)	\	abs(HServoPos(cTibiaPin(LegIndex)) - aswTibiaServo(LegIndex)) * 20 / SSCTime]
    next
  endif
  return

#endif ;ARC32_SERVO_DRIVER	