#ifndef ARC32_SERVO_DRIVER
;Project Lynxmotion Phoenix
;Description: Phoenix software
;Software version: V2.1 alfa
;Date: 15-04-2011
;Programmer: Jeroen Janssen (aka Xan)
;
; SSC Driver - Contains all of the code that knows anything about the SSC-32
; 		It also has timer functions as this may also change with drivers...
;
;====================================================================
lTimerCnt			var long	

;--------------------------------------------------------------------
;[TIMER INTERRUPT INIT]
#ifdef BASICATOMPRO28
TIMERINT	con	TIMERAINT
ONASMINTERRUPT TIMERAINT, HANDLE_TIMERA_ASM 
#else
TIMERINT	con	TIMERB1INT
ONASMINTERRUPT TIMERB1INT, HANDLE_TIMERB1_ASM 
#endif

;--------------------------------------------------------------------
;[InitServoDriver] - Initializes the servo driver including the main timer used in the phoenix code
InitServoDriver:
;Timer
; Timer A init, used for timing of messages and some times for timing code...
#ifdef BASICATOMPRO28
  WTIMERTICSPERMSMUL con 64	; BAP28 is 16mhz need a multiplyer and divider to make the conversion with /8192
  WTIMERTICSPERMSDIV con 125  ; 
  TMA = 0	; clock / 8192					; Low resolution clock - used for timeouts...
   ENABLE TIMERINT
#else
  WTIMERTICSPERMSMUL con 256	; Arc32 is 20mhz need a multiplyer and divider to make the conversion with /8192
  WTIMERTICSPERMSDIV con 625  ; 
  TMB1 = 0	; clock / 8192					; Low resolution clock - used for timeouts...
  ENABLE TIMERB1INT
#endif

  return

;==============================================================================
;[Handle_Timer_asm] - Handle timer A overlfow in assembly language.  Currently only
;used for timings for debuging the speed of the code
;Now used to time how long since we received a message from the remote.
;this is important when we are in the NEW message mode, as we could be hung
;out with the robot walking and no new commands coming in.
;==============================================================================
#ifdef BASICATOMPRO28
   BEGINASMSUB 
HANDLE_TIMERA_ASM 
	push.l 	er1                  ; first save away ER1 as we will mess with it. 
	bclr 	#6,@IRR1:8               ; clear the cooresponding bit in the interrupt pending mask 
	mov.l 	@LTIMERCNT:16,er1      ; Add 256 to our counter 
	add.l	#256,er1 
	mov.l 	er1, @LTIMERCNT:16 
	pop.l 	er1 
	rte 
	ENDASMSUB 
#else
   BEGINASMSUB 
HANDLE_TIMERB1_ASM 
	push.l 	er1                  ; first save away ER1 as we will mess with it. 
	bclr 	#5,@IRR2:8               ; clear the cooresponding bit in the interrupt pending mask 
	mov.l 	@LTIMERCNT:16,er1      ; Add 256 to our counter 
	add.l	#256,er1 
	mov.l 	er1, @LTIMERCNT:16 
	pop.l 	er1 
	rte 
	ENDASMSUB 
#endif
	return		; Put a basic statement before...

;==============================================================================
;[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
;                makes sure of consistancy. That is it is very posible that 
;                after we grabed the timers value it overflows, before we grab the other part
;                so we check to make sure it is correct and if necesary regrab things.
;==============================================================================
GetCurrentTime:
#ifdef BASICATOMPRO28
  lCurrentTime = lTimerCnt + TCA
#else	
  lCurrentTime = lTimerCnt + TCB1
#endif	
  ; handle wrap
  if lTimerCnt <> (lCurrentTime & 0xffffff00) then
#ifdef BASICATOMPRO28	
	lCurrentTime = lTimerCnt + TCA
#else
	lCurrentTime = lTimerCnt + TCB1
#endif
  endif

  return lCurrentTime


;-------------------------------------------------------------------------------------
;[ConvertTimeMS]
_ttconv	var	long
ConvertTimeMS[_ttconv]:
	return (_ttconv * WTIMERTICSPERMSMUL)/WTIMERTICSPERMSDIV 


;--------------------------------------------------------------------

;[CheckGPEnable] Checks to see if the SSC-32 support the general purpose sequences
CheckGPEnable:
	
  pause 10
  GPEnable=0
  serout cSSC_OUT, cSSC_BAUD, ["ver", 13]
	
GetSSCVersion:
  serin cSSC_IN, cSSC_BAUD, 10000, timeout, [GPVerData(Index)]
  Index = (Index+1)//3 ; shift last 3 chars in data
  goto GetSSCVersion
	
timeout:
  if (GPVerData(0) + GPVerData(1) + GPVerData(2)) = 164 then ; Check if the last 3 chars are G(71) P(80) cr(13)
	return 1
  else
  	sound cSpeakerPin, [40\5000,40\5000]
  endif
  return 0

;--------------------------------------------------------------------
;[GP PLAYER]
GPStatSeq       var byte
GPStatFromStep    var byte
GPStatToStep   var byte
GPStatTime      var byte
GPPlayer:

  ;Start sequence
  IF (GPStart=1) THEN
    disable TIMERINT 		;disable timer interrupt
    serout cSSC_OUT, cSSC_BAUD, ["PL0SQ", dec GPSeq, "ONCE", 13] ;Start sequence
    enable TIMERINT
    GPStart=2   ; set the state to do query...
  ELSEIF (GPStart=2)
    ;See if  GPPlayer has completed sequence
    disable TIMERINT 		;disable timer interrupt
    serout cSSC_OUT, cSSC_BAUD, ["QPL0", 13]
    serin cSSC_IN, cSSC_BAUD, [GPStatSeq, GPStatFromStep, GPStatToStep, GPStatTime]
    enable TIMERINT
    IF (GPStatSeq=255) and  (GPStatFromStep=0) and (GPStatToStep=0) and (GPStatTime=0) THEN
        GPStart=0
    ENDIF
  ELSEIF GPStart=0xff
    ; user requested us to abort the sequence.
    disable TIMERINT 		;disable timer interrupt
    serout cSSC_OUT, cSSC_BAUD, ["PL0", 13] ;Stop the sequence now
    enable TIMERINT
    GPStart = 0
  ENDIF
return
	
	
;--------------------------------------------------------------------
;[SERVO DRIVER] Updates the positions of the servos
;Binary version
;**********************************************************************
;Extended binary commands.
;
;Commands 0x80-0x9F are group move servo number 0-31.  The next 2 bytes
;must be pulse width for the servo:
;    0x80+servoNum, pwHigh, pwLow
;
;Command 0xA0 is the group move servo speed, which must immediately
;follow the pulse width:
;    0xA0, spdHigh, spdLow
;
;Command 0xA1 is the group move time, which must follow all of the
;pulse widths and speeds:
;    0xA1, timeHigh, timeLow
;
;Command 0xA2 is the stop all command, 1 byte
;    0xA2
;**********************************************************************
UpdateServoDriver:

  disable TIMERINT 		;disable timer interrupt
  for LegIndex = 0 to 5
    ; Calulate 
	GOSUB GetPWMValues [LegIndex]
	
    serout cSSC_OUT, cSSC_BAUD, [0x80+cCoxaPin(LegIndex) ,CoxaPWM.highbyte,  CoxaPWM.lowbyte,|
    	                         0x80+cFemurPin(LegIndex),FemurPWM.highbyte, FemurPWM.lowbyte,|
    	                         0x80+cTibiaPin(LegIndex),TibiaPWM.highbyte, TibiaPWM.lowbyte]
  next
  
  ;Send speed
  serout cSSC_OUT, cSSC_BAUD, [0xA1,SSCTime.highbyte,SSCTime.lowbyte]
  enable TIMERINT

  PrevSSCTime = SSCTime
return
;--------------------------------------------------------------------
;[FREE SERVOS] Frees all the servos (binary)
FreeServos
    disable TIMERINT
	for LegIndex = 0 to 31
      serout cSSC_OUT, cSSC_BAUD, [0x80+LegIndex, 0, 0]
    next
    serout cSSC_OUT, cSSC_BAUD, [0xA1, 0, 200]
    enable TIMERINT
return
;--------------------------------------------------------------------
;[COMMIT SERVO POSITIONS]
CommitServoDriver:
  ;Send <CR>
  disable TIMERINT
  serout cSSC_OUT, cSSC_BAUD, [13]	
  enable TIMERINT
return

#endif ; ARC32_PHOENIX_DRIVER
	